/*  plumb - the pipe shell - set up and maintain plumb timers
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "plumb.h"
#include "_default.h"
#include "timer.h"
#include "pipe.h"
#include "virt.h"

typedef struct vp_timer_s {
	/* config */
	int repeat;
	double period;
	char *msg;
	int msg_len;
	int event;

	/* current state */
	int counter;
} vp_timer_t;


static virt_ctrl_t timer_timer(proc_virt_t *proc, void *data)
{
	vp_timer_t *t = proc->data;
	char *buff;
	int len;

	buff = malloc(t->msg_len + 64);
/* if we have an msg, write that to stdout */
	if (*t->msg != '\0')
		vp_write(proc, vstdout, t->msg, t->msg_len);

/* if we have a timer or we didn't have msg, prepare the long message */
	if ((t->event) || (*t->msg == '\0'))
		len = sprintf(buff, "plumb_timer %d %s\n", t->counter-1, t->msg);

/* we didn't have msg, write the long message */
	if (*t->msg == '\0')
		vp_write(proc, vstdout, buff, len);

/* if we have event, write the long message */
	if (t->event)
		vp_write(proc, vstdevent, buff, len);

	free(buff);

	if ((t->counter == -1) || ((--t->counter) > 0)) {
		timer_add_delta(proc, t->period, NULL);
		return VC_OK;
	}
	return VC_QUIT;
}

static virt_ctrl_t timer_start(proc_virt_t *p)
{
	vp_timer_t *t = p->data;
	t->counter = t->repeat;
	if ((t->counter > 0) || (t->counter == -1))
		timer_add_delta(p, t->period, NULL);
	return VC_OK;
}

virt_ctrl_t timer_source_block(proc_virt_t *proc, int local_fd, int is_blocking)
{
	return VC_OK;
}

static void timer_free(proc_virt_t *p)
{
	vp_timer_t *t = p->data;
	
	free(t->msg);
	free(t);
}

int vp_timer_init(proc_virt_t *p, kvlist_t *args)
{
	kv_t *arg;
	vp_timer_t *t;

	vp_stdfds_set(p, VP_REQ_UNBOUND, VP_REQ_ANY, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	/* allocate a process descriptor */
	p->input = vp_input_discard;
	p->sink_eof = NULL;
	p->source_eof = NULL;
#warning TODO: we should not emit messages while source is blocked - use some buffer?
	p->source_block = NULL;
	p->timer = timer_timer;
	p->start = timer_start;
	p->free = timer_free;

	t = calloc(sizeof(vp_timer_t), 1);
	for(arg = args->head; arg != NULL; arg = arg->next)
	{
		switch(*arg->key) {
			case 'r': t->repeat = atoi(arg->value); break;  /* repeat */
			case 'p': t->period = atof(arg->value); break;  /* period */
			case 'm': t->msg    = arg->value; arg->value=NULL; break;  /* msg */
			case 'e': t->event  = 1;                break;  /* event */
			default: plumb_error("Unkown argument %s for [timer]\n", arg->key);
		}
	}

	if (t->msg == NULL)
		t->msg = strclone("");
	if (t->event)
		vp_pipe_stdevent(p);
	if (t->repeat <= 0)
		t->repeat = -1;
	t->msg_len = strlen(t->msg);
	p->data = t;

	return 0;
}
